#!/usr/bin/env python3
import streamlink
import tornado.web
import tornado.routing
import requests
import os

providers = {}
providers["nrk"] = "https://tv.nrk.no"
providers["svt"] = "https://svtplay.se"
providers["youtube"] = "https://youtu.be"
providers["twitch"] = "https://twitch.tv"

proxies = {}
proxies["nrk"] = streamlink.Streamlink()
proxies["svt"] = streamlink.Streamlink()
proxies["youtube"] = streamlink.Streamlink()
proxies["twitch"] = streamlink.Streamlink()

for key in proxies:
    proxy = os.environ.get(key.upper() + "_PROXY")
    proxies[key].set_option("http-timeout", 2.0)
    if proxy is not None:
        proxies[key].set_option("https-proxy", proxy)

class MainHandler(tornado.web.RequestHandler):
    def get(self):
        provider = self.get_query_argument("provider", None)
        endpoint = None
        if provider is not None and provider in providers.keys():
            proxy = proxies.get(provider)
            src = providers[provider] + self.request.uri
            try:
                resp = requests.get(src)
                if resp is not None:
                    src = resp.url
            except Exception as e:
                self.write(e)
                return
            else:
                try:
                    streams = proxy.streams(src)
                    for key in reversed(streams):
                        stream = streams.get(key)
                        if hasattr(stream, "url"):
                            endpoint = stream.url
                            break
                except Exception as e:
                    self.write(str(e))
                    return
        if endpoint is None:
            self.write("stream not found")
        else:
            self.redirect(endpoint, status=303)
try:
    app_web = tornado.web.Application([(tornado.routing.AnyMatches(), MainHandler)])
    app_web.listen(8080)
    tornado.ioloop.IOLoop.current().start()
except KeyboardInterrupt:
    print()

